@@ -21,6 +21,7 @@ from . import config
 
 # Module API
 
+# TODO: merge pick/skip rows logic
 class Stream(object):
     """Stream of tabular data.
 
@@ -97,6 +98,9 @@ class Stream(object):
             parsing malformed rows, simply returning an empty value. Defaults
             to False.
 
+        pick_rows (List[Union[int, str, dict]], optional):
+            The same as `skip_rows` but it's for picking rows instead of skipping.
+
         skip_rows (List[Union[int, str, dict]], optional):
             List of row numbers, strings and regex patterns as dicts to skip.
             If a string, it'll skip rows that their first cells begin with it e.g. '#' and '//'.
@@ -104,13 +108,13 @@ class Stream(object):
             To provide a regex pattern use  `{'type'\\: 'regex', 'value'\\: '^#'}`
             For example\\: `skip_rows=[1, '# comment', {'type'\\: 'regex', 'value'\\: '^# (regex|comment)'}]`
 
+        pick_columns (str[]):
+            Alias for `ignore_not_listed_headers`.
+
         skip_columns (str[]):
             Alias for `ignore_listed_headers`.
             If it contains an empty string it will activate `ignore_blank_headers`
 
-        pick_columns (str[]):
-            Alias for `ignore_not_listed_headers`.
-
         post_parse (List[function], optional):
             List of generator functions that
             receives a list of rows and headers, processes them, and yields
@@ -153,15 +157,24 @@ class Stream(object):
                  multiline_headers_joiner=' ',
                  force_strings=False,
                  force_parse=False,
-                 skip_rows=[],
-                 skip_columns=None,
+                 pick_rows=None,
+                 skip_rows=None,
                  pick_columns=None,
+                 skip_columns=None,
                  post_parse=[],
                  custom_loaders={},
                  custom_parsers={},
                  custom_writers={},
                  **options):
 
+        # Translate aliases
+        if pick_columns is not None:
+            ignore_not_listed_headers = pick_columns
+        if skip_columns is not None:
+            ignore_listed_headers = skip_columns
+            if '' in skip_columns:
+                ignore_blank_headers = True
+
         # Set headers
         self.__headers = None
         self.__headers_row = None
@@ -178,20 +191,32 @@ class Stream(object):
             else:
                 self.__headers = list(headers)
 
-        # Translate aliases
-        if pick_columns is not None:
-            ignore_not_listed_headers = pick_columns
-        if skip_columns is not None:
-            ignore_listed_headers = skip_columns
-            if '' in skip_columns:
-                ignore_blank_headers = True
+        # Set pick rows
+        self.__pick_rows = pick_rows
+        self.__pick_rows_by_numbers = []
+        self.__pick_rows_by_patterns = []
+        self.__pick_rows_by_comments = []
+        self.__pick_rows_by_presets = {}
+        for directive in copy(pick_rows or []):
+            if isinstance(directive, int):
+                self.__pick_rows_by_numbers.append(directive)
+            elif isinstance(directive, dict):
+                if directive['type'] == 'regex':
+                    self.__pick_rows_by_patterns.append(re.compile(directive['value']))
+                elif directive['type'] == 'preset' and directive['value'] == 'blank':
+                    self.__pick_rows_by_presets['blank'] = True
+                else:
+                    raise ValueError('Not supported pick rows: %s' % directive)
+            else:
+                self.__pick_rows_by_comments.append(str(directive))
 
         # Set skip rows
+        self.__skip_rows = skip_rows
         self.__skip_rows_by_numbers = []
         self.__skip_rows_by_patterns = []
         self.__skip_rows_by_comments = []
         self.__skip_rows_by_presets = {}
-        for directive in copy(skip_rows):
+        for directive in copy(skip_rows or []):
             if isinstance(directive, int):
                 self.__skip_rows_by_numbers.append(directive)
             elif isinstance(directive, dict):
@@ -231,6 +256,7 @@ class Stream(object):
         self.__actual_scheme = scheme
         self.__actual_format = format
         self.__actual_encoding = encoding
+        self.__actual_compression = compression
         self.__options = options
         self.__sample_extended_rows = []
         self.__loader = None
@@ -346,6 +372,11 @@ class Stream(object):
         parser_class = self.__custom_parsers.get(format)
         if parser_class is None:
             if format not in config.PARSERS:
+                # If not existent it's a not-found error
+                # Next line will raise IOError/HTTPError
+                chars = self.__loader.load(source)
+                chars.close()
+                # Otherwise it's a format error
                 message = 'Format "%s" is not supported' % format
                 raise exceptions.FormatError(message)
             parser_class = helpers.import_attribute(config.PARSERS[format])
@@ -371,6 +402,7 @@ class Stream(object):
         self.__actual_scheme = scheme
         self.__actual_format = format
         self.__actual_encoding = self.__parser.encoding
+        self.__actual_compression = compression or 'no'
 
         return self
 
@@ -390,6 +422,16 @@ class Stream(object):
             self.__extract_headers()
         self.__row_number = 0
 
+    @property
+    def source(self):
+        """Source
+
+        # Returns
+            any: stream source
+
+        """
+        return self.__source
+
     @property
     def headers(self):
         """Headers
@@ -452,6 +494,16 @@ class Stream(object):
         """
         return self.__actual_encoding
 
+    @property
+    def compression(self):
+        """Stream's compression ("no" if no compression)
+
+        # Returns
+            str: compression
+
+        """
+        return self.__actual_compression
+
     @property
     def size(self):
         """Returns the BYTE count of the read chunks if available
@@ -533,15 +585,21 @@ class Stream(object):
         iterator = self.__apply_processors(iterator)
 
         # Yield rows from iterator
-        for row_number, headers, row in iterator:
-            if row_number > self.__row_number:
-                self.__row_number = row_number
-                if extended:
-                    yield (row_number, headers, row)
-                elif keyed:
-                    yield dict(zip(headers, row))
-                else:
-                    yield row
+        try:
+            for row_number, headers, row in iterator:
+                if row_number > self.__row_number:
+                    self.__row_number = row_number
+                    if extended:
+                        yield (row_number, headers, row)
+                    elif keyed:
+                        yield dict(zip(headers, row))
+                    else:
+                        yield row
+        except UnicodeError as error:
+            message = 'Cannot parse the source "%s" using "%s" encoding at "%s"'
+            raise exceptions.EncodingError(message % (self.__source, error.encoding, error.start))
+        except Exception as error:
+            raise exceptions.SourceError(str(error))
 
     def read(self, keyed=False, extended=False, limit=None):
         """Returns a list of rows.
@@ -568,7 +626,7 @@ class Stream(object):
     def save(self, target, format=None,  encoding=None, **options):
         """Save stream to the local filesystem.
 
-        # Arguments:
+        # Arguments
             target (str): Path where to save the stream.
             format (str, optional):
                 The format the stream will be saved as. If
@@ -577,6 +635,8 @@ class Stream(object):
                 Saved file encoding. Defaults to ``config.DEFAULT_ENCODING``.
             **options: Extra options passed to the writer.
 
+        # Returns
+            count (int?): Written rows count if available
         """
 
         # Get encoding/format
@@ -602,7 +662,7 @@ class Stream(object):
 
         # Write data to target
         writer = writer_class(**writer_options)
-        writer.write(self.iter(), target, headers=self.headers, encoding=encoding)
+        return writer.write(self.iter(), target, headers=self.headers, encoding=encoding)
 
     # Private
 
@@ -624,6 +684,11 @@ class Stream(object):
                 self.__sample_extended_rows.append((row_number, headers, row))
             except StopIteration:
                 break
+            except UnicodeError as error:
+                message = 'Cannot parse the source "%s" using "%s" encoding at "%s"'
+                raise exceptions.EncodingError(message % (self.__source, error.encoding, error.start))
+            except Exception as error:
+                raise exceptions.SourceError(str(error))
 
     def __extract_headers(self):
 
@@ -778,30 +843,69 @@ class Stream(object):
 
     def __check_if_row_for_skipping(self, row_number, headers, row):
 
-        # Skip by number
-        if row_number in self.__skip_rows_by_numbers:
+        # Pick rows
+        if self.__pick_rows:
+
+            # Skip by number
+            if row_number in self.__pick_rows_by_numbers:
+                return True
+
+            # Get first cell
+            cell = row[0] if row else None
+
+            # Handle blank cell/row
+            if cell in [None, '']:
+                if '' in self.__pick_rows_by_comments:
+                    return False
+                if self.__pick_rows_by_presets.get('blank'):
+                    if not list(filter(lambda cell: cell not in [None, ''], row)):
+                        return False
+                return True
+
+            # Pick by pattern
+            for pattern in self.__pick_rows_by_patterns:
+                if bool(pattern.match(cell)):
+                    return False
+
+            # Pick by comment
+            for comment in filter(None, self.__pick_rows_by_comments):
+                if cell.startswith(comment):
+                    return False
+
+            # Default
             return True
 
-        # Get first cell
-        cell = row[0] if row else None
+        # Skip rows
+        if self.__skip_rows:
 
-        # Handle blank cell/row
-        if cell in [None, '']:
-            if '' in self.__skip_rows_by_comments:
+            # Skip by number
+            if row_number in self.__skip_rows_by_numbers:
                 return True
-            if self.__skip_rows_by_presets.get('blank'):
-                if not list(filter(lambda cell: cell not in [None, ''], row)):
+
+            # Get first cell
+            cell = row[0] if row else None
+
+            # Handle blank cell/row
+            if cell in [None, '']:
+                if '' in self.__skip_rows_by_comments:
                     return True
+                if self.__skip_rows_by_presets.get('blank'):
+                    if not list(filter(lambda cell: cell not in [None, ''], row)):
+                        return True
+                return False
+
+            # Skip by pattern
+            for pattern in self.__skip_rows_by_patterns:
+                if bool(pattern.match(cell)):
+                    return True
+
+            # Skip by comment
+            for comment in filter(None, self.__skip_rows_by_comments):
+                if cell.startswith(comment):
+                    return True
+
+            # Default
             return False
 
-        # Skip by pattern
-        for pattern in self.__skip_rows_by_patterns:
-            if bool(pattern.match(cell)):
-                return True
-
-        # Skip by comment
-        for comment in filter(None, self.__skip_rows_by_comments):
-            if cell.startswith(comment):
-                return True
-
+        # No pick/skip
         return False
